-- ========================================================================== --
--                    Boss Mod Nameplates                                     --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/boss-mod-nameplates        --
-- ========================================================================== --
Scorpio                   "BossModNameplates"                            "1.2.2"
-- ========================================================================== --
namespace "BMN"
-- ==========================[[ Logger ]]=======================================
Log               = Logger("BossModNameplates")

Trace             = Log:SetPrefix(1, "|cffa9a9a9[BossModNameplates:Trace]|r", true)
Debug             = Log:SetPrefix(2, "|cff808080[BossModNameplates:Debug]|r", true)
Info              = Log:SetPrefix(3, "|cffffffff[BossModNameplates:Info]|r", true)
Warn              = Log:SetPrefix(4, "|cffffff00[BossModNameplates:Warn]|r", true)
Error             = Log:SetPrefix(5, "|cffff0000[BossModNameplates:Error]|r", true)
Fatal             = Log:SetPrefix(6, "|cff8b0000[BossModNameplates:Fatal]|r", true)

Log.LogLevel = 3
Log:AddHandler(print)
-- =============================================================================
-- Get the current scorpio version of player.
_SCORPIO_VERSION = tonumber(GetAddOnMetadata("Scorpio", "Version"):match("%d+$"))


function OnLoad(self)
  _DB = SVManager("BossModNameplatesDB")

  _DB:SetDefault("General", {
    bigWigsSupport = true,
    dbmSupport = true,
  })

  _DB:SetDefault("Nameplates", {
    height = 64,
    width = 64,
    showBorder = true,
    offsetX = 0,
    offsetY = -10,
    borderColor = { r = 0, g = 0, b = 0},
    anchorFrom = "BOTTOM",
    anchorTo = "TOP",
    simulateOverlapMotionOnFriendly = false,
    clickThroughtFriendly = false,
    clickThroughtEnemy = false,
    clickThroughtSelf = false,
  })

  -- Apply the DB to Nameplate
  Nameplate.defaultWidth = _DB.Nameplates.width
  Nameplate.defaultHeight = _DB.Nameplates.height
  Nameplate.defaultOffsetX = _DB.Nameplates.offsetX
  Nameplate.defaultOffsetY = _DB.Nameplates.offsetY
  Nameplate.defaultAnchorFrom = _DB.Nameplates.anchorFrom
  Nameplate.defaultAnchorTo = _DB.Nameplates.anchorTo

  self.unitsCache = {}
  self.auras = Dictionary()

  C_NamePlate.SetNamePlateFriendlyClickThrough(_DB.Nameplates.clickThroughtFriendly)
  C_NamePlate.SetNamePlateEnemyClickThrough(_DB.Nameplates.clickThroughtEnemy)
  C_NamePlate.SetNamePlateSelfClickThrough(_DB.Nameplates.clickThroughtSelf)

end

function OnEnable(self)
    if _DB.Nameplates.simulateOverlapMotionOnFriendly then
      self:EnableSimulateOverlapMotionOnFriendly()
    end
end

function OnDisable(self)

end


-- ========================================================================== --
-- Unit/Aura Methods
-- ========================================================================== --

-- Get the unit from guid. This method use two method :
-- First, it searchs in the cache (created when the encounter hass started)
-- but it works only for the raid & party members.
-- if not found, it searchs in the nameplates working in any situation but less
-- efficient
function GetUnit(self, guid)
  return self.unitsCache[guid] or self:GetUnitFromNameplates(guid)
end

-- Get the unit from nameplates, less efficient that get unit from cache
-- but it works in any situation.
-- return the unit found and the nameplate frame.
function GetUnitFromNameplates(self, guid)
  for _, frame in pairs(C_NamePlate.GetNamePlates()) do
    local unit = self:GetUnitByFrame(frame)
    if unit and UnitGUID(unit) == guid then
      return unit, frame
    end
  end
end

function BuildUnitsCache(self, size)
  if IsInRaid() then
    for i = 1, size do
      local unit = "raid"..i
      local guid = UnitGUID(unit)
      if guid then
        self.unitsCache[guid] = unit
      end
    end
  elseif IsInGroup() then
    for i = 1, size do
      local unit = "party"..i
      local guid = UnitGUID(unit)
      if guid then
        self.unitsCache[guid] = unit
      end
    end
  end
end

function ClearUnitsCache(self)
   wipe(self.unitsCache)
end


-- Get a valid unit from nameplate frame, it use to add the compatibility
-- with some addon like Elvui.
function GetUnitByFrame(self, frame)
    if frame.namePlateUnitToken then
      return frame.namePlateUnitToken
    -- @NOTE : Needed for the ElVUI users
    elseif frame.UnitFrame and frame.UnitFrame.unit then
      return frame.UnitFrame.unit
    end
end

-- A method helper to get the aura duration from spell on a unit called by his guid
-- example : GetAuraDuration(UnitGUID("target"), "Searing Brand") or GetAuraDuration(unitGUID, 213166)
-- It returns the duration remaining and the maximun duration.
function GetAuraDuration(self, guid, spell)
  if not guid or not spell then return end

  local unit = self:GetUnit(guid)
  if unit then
    local spellName = type(spell) == "number" and GetSpellInfo(spell) or spell

    local maxDuration, endTime
    if UnitDebuff(unit, spellName) then
      maxDuration, endTime = select(6, UnitDebuff(unit, spellName))
    elseif UnitBuff(unit, spellName) then
      maxDuration, endTime = select(6, UnitBuff(unit, spellName))
    end

    --print(endTime and endTime - GetTime(), maxDuration, spellName)
    return endTime and endTime - GetTime(), maxDuration
  end
end
-- ========================================================================== --

__SystemEvent__()
function NAME_PLATE_CREATED(frame)
  Nameplate(frame)
  --frame.dtext = frame:CreateTexture()
  --frame.dtext:SetAllPoints()
  --frame.dtext:SetTexture(1, 0, 0)
end

__SystemEvent__()
function NAME_PLATE_UNIT_ADDED(unit)
  local frame = C_NamePlate.GetNamePlateForUnit(unit)
  local guid = UnitGUID(unit)
  local aura = _Addon.auras[guid]

  if aura and frame.BMN then
    frame.BMN:UpdateFromAura(aura)
    frame.BMN:Show()
  end
end


__SystemEvent__()
function NAME_PLATE_UNIT_REMOVED(unit)
  local frame = C_NamePlate.GetNamePlateForUnit(unit)

  if frame.BMN then
    frame.BMN:Hide()
  end
end

__SystemEvent__()
function PLAYER_REGEN_DISABLED()
  if _DB.Nameplates.simulateOverlapMotionOnFriendly then
    _Addon:EnableSimulateOverlapMotionOnFriendly()
  end
end


__SystemEvent__()
function ENCOUNTER_START(encounterID, encounterName, difficulty, size)
  Trace("ENCOUNTER_START : %i, %s, %i, %i", encounterID, encounterName, difficulty, size)
  _Addon.isInEncounter = true
  _Addon.encounterID = encounterID
  _Addon.encounterName = encounterName
  _Addon.encounterDifficulty = difficulty
  _Addon.encounterSize = size

  _Addon:BuildUnitsCache(size)
end

__SystemEvent__()
function ENCOUNTER_END(encounterID, encounterName, difficulty, size, endStatus)
  _Addon.isInEncounter = false
  _Addon:ClearNameplateAuras()

  _Addon:ClearUnitsCache()
end

-- ========================================================================== --
-- Nameplate function
-- ========================================================================== --
function ShowNameplateAura(self, aura)
  if not aura then
    Debug("The aura isn't valid, it cannot be shown")
    return
  end

  self.auras[aura.guid] = aura

  local frame
  if aura.unit then
    frame = C_NamePlate.GetNamePlateForUnit(aura.unit)
  else
    _, frame = self:GetUnitFromNameplates(aura.guid)
  end

  if frame and frame.BMN then
    frame.BMN:UpdateFromAura(aura)
    frame.BMN:Show()
  end
end

function HideNameplateAura(self, guid)
  local aura = self.auras[guid]

  if not aura then return end

  local frame
  if aura.unit then
    frame = C_NamePlate.GetNamePlateForUnit(aura.unit)
  else
    _, frame = self:GetUnitFromNameplates(aura.guid)
  end

  if frame and frame.BMN then
    frame.BMN:Hide()
  end

  self.auras[guid] = nil
  aura = nil
end

function ClearNameplateAuras(self)
  for guid, _ in self.auras:GetIterator() do
    self:HideNameplateAura(guid)
  end
end

local previousValue

__BigWigsEvent__ "BigWigs_EnableFriendlyNameplates"
__DBMEvent__ "BossMod_DisableFriendlyNameplates"
function EnableFriendlyNameplates()
  NoCombat(function()
    Debug("Enable Friendly Nameplates")
    previousValue = GetCVar("nameplateShowFriends")
    SetCVar("nameplateShowFriends",1)
  end)
end


__BigWigsEvent__ "BigWigs_DisableFriendlyNameplates"
__DBMEvent__ "BossMod_DisableFriendlyNameplates"
function DisableFriendlyNameplates()
  NoCombat(function()
    Debug("Disable friendly nameplates")
    SetCVar("nameplateShowFriends", previousValue)
  end)
end

-- ========================================================================== --
-- Options
-- ========================================================================== --
__SlashCmd__ "bmn" "config" "- open the options"
function LoadOptions()
  if IsAddOnLoaded("BossModNameplates_Options") then return end

  local loaded, reason = LoadAddOn("BossModNameplates_Options")

  if not loaded then
    Error(string.format("The options cannot be loaded (reason : %s)", reason))
  end
end


local inConfigMode = false
__SlashCmd__ "bmn" "configmode" "- toggle the config mode"
function ToggleConfigMode()
  inConfigMode = not inConfigMode

  if inConfigMode then _Addon:StartConfigMode() else _Addon:StopConfigMode() end

end

function StopConfigMode()
  _Addon:ClearNameplateAuras()
end

function StartConfigMode()
  _Addon:ClearNameplateAuras()
  local spells = List(205429, 205445, 216345, 216344, 227498, 227491, 227500, 227499)

  for _, frame in pairs(C_NamePlate.GetNamePlates()) do
    local unit = _Addon:GetUnitByFrame(frame)
    if UnitIsFriend("player", unit) and UnitIsPlayer(unit) then
      local aura = Aura()
      aura.unit = unit
      aura.guid = UnitGUID(unit)
      aura.icon = GetSpellTexture(spells[math.random(1, spells.Count)])
      aura.duration = 60
      aura.startTime = GetTime()
      aura.endTime = aura.startTime + aura.duration
      aura.desaturated = math.random(1, 2) == 1 and true or false
      aura.borderColor = { r = 0, g = 0, b = 0}

      _Addon:ShowNameplateAura(aura)
    end
  end

end

local oldFriendlyWidth, oldFriendlyHeight = C_NamePlate.GetNamePlateFriendlySize()
function EnableSimulateOverlapMotionOnFriendly(self)
  C_NamePlate.SetNamePlateFriendlySize(1, 1)
end

function DisableSimulateOverlapMotionOnFriendly(self)
  C_NamePlate.SetNamePlateFriendlySize(oldFriendlyWidth, oldFriendlyHeight)
end


local toggleBackdropTest = false
function ToggleBackdropTest(self)
  if toggleBackdropTest then
    self:EnableBackdropTest()
  else
    self:DisableBackdropTest()
  end

  toggleBackdropTest = not toggleBackdropTest
end


function EnableBackdropTest()
  for _, frame in pairs(C_NamePlate.GetNamePlates()) do
    frame.dbtex:Show()
  end
end

function DisableBackdropTest()
  for _, frame in pairs(C_NamePlate.GetNamePlates()) do
    frame.dbtex:Hide()
  end
end

-- ========================================================================== --
-- Debug
-- ========================================================================== --
__Thread__() __SlashCmd__ "bmn" "debugaura"
function DebugTestAura(arg, ...)
  Debug("Add Aura for test")
  local spell = type(arg) == "string" and tonumber(arg)

  local aura = Aura()
  aura.unit = "target"
  aura.guid = UnitGUID("target")
  aura.duration = 10
  aura.desaturated = true
  --aura.icon = GetSpellTexture(205429)
  aura.icon =  spell and GetSpellTexture(spell) or GetSpellTexture(205429)
  aura.startTime = GetTime()
  aura.endTime = aura.startTime + aura.duration
  aura.borderColor = { r = 0, g = 0, b =  0 }


  Encounter.ApplyCustomChange(aura,
      Encounter.Legion.Nighthold.Etraeus,
      Difficulty.Mythic,
      BossMod.BigWigs)

  --if desaturate then
    --aura.borderColor = { r = 1, g = 0, b = 0}
  --else
    --aura.borderColor = { r = 0, g = 1, b = 0 }
  --end

  _Addon:ShowNameplateAura(aura)

  Delay(aura.duration)
  _Addon:HideNameplateAura(aura.guid)
  Debug("Remove Aura for test")

end

-- ========================================================================== --
-- Other Command
-- ========================================================================== --
__SlashCmd__ "bmn" "scorpio" "- return the current scorpio version"
function ReturnScorpioVersion()
  Info("|cff00ff00Your scorpio version is:|r |cffff0000%s|r", GetAddOnMetadata("Scorpio", "Version"))
end
