-- ========================================================================== --
--                       Boss Mod Nameplates                                  --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/boss-mod-nameplates        --
-- ========================================================================== --
Scorpio             "BossModNameplates.Options"                          "1.2.2"
-- ========================================================================== --
namespace  "BMN"
-- ========================================================================== --
_AceGUI = LibStub("AceGUI-3.0")
_AceConfig = LibStub("AceConfig-3.0")
_AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
_AceConfigDialog = LibStub("AceConfigDialog-3.0")
-- ========================================================================== --
local f

_AnchorPoints = {
  ["TOP"] = "TOP",
  ["TOPLEFT"] = "TOPLEFT",
  ["TOPRIGHT"] = "TOPRIGHT",
  ["BOTTOM"] = "BOTTOM",
  ["BOTTOMLEFT"] = "BOTTOMLEFT",
  ["BOTTOMRIGHT"] = "BOTTOMRIGHT",
  ["LEFT"] = "LEFT",
  ["RIGHT"] = "RIGHT",
  ["CENTER"] = "CENTER",
}

_Settings = {
  type = "group",
  name = "Boss Mod Nameplates - Options",
  childGroups = "tab",
  args = {
    general = {
      type = "group",
      name = "General",
      order = 1,
      args = {
          bossModSupport = {
            type = "group",
            name = "Boss Mod Support",
            inline = true,
            order = 1,
            disabled = true,
            args = {
              description = {
                type = "description",
                name = [[Enable the support for the following Boss mods. The addon will ignore all boss mods that are unchecked.]],
                order = 1,
              },
              bigWigs = {
                type = "toggle",
                name = "BigWigs",
                order = 2,
                get = function() return _DB.General.bigWigsSupport end,
                set = function(_, support) _DB.General.bigWigsSupport = support end,
              },
              dbm = {
                type = "toggle",
                name = "DBM",
                order = 3,
                get = function() return _DB.General.dbmSupport end,
                set = function(_, support) _DB.General.dbmSupport = support end,
              }
            }
          }
      }
    },
    nameplates = {
      type = "group",
      name = "Nameplates",
      order = 2,
      childGroups = "tab",
      args = {
        general = {
          type = "group",
          name = "Position & Size",
          order = 1,
          args = {
            configGroup = {
              type = "group",
              name = "Config Mode",
              inline = true,
              order = 1,
              args = {
                description = {
                  type = "description",
                  name = [[The config mode will create fake random icons (desaturated or not) on the friendly players. It allows to see the result of your settings.]],
                  order = 1,
                },
                configMode = {
                  type = "execute",
                  name = "Toggle Config Mode",
                  order = 2,
                  func = function() _Addon:ToggleConfigMode() end,
                }
              }
            },
            size = {
              type = "group",
              name = "Size",
              inline = true,
              order = 2,
              args = {
                width = {
                  type = "range",
                  name = "Width",
                  order = 1,
                  step = 1,
                  min = 8,
                  max = 128,
                  get = function() return _DB.Nameplates.width end,
                  set = function(_, width)
                    _DB.Nameplates.width = width ; Nameplate.defaultWidth = width
                  end,
                },
                height = {
                  type = "range",
                  name = "Height",
                  order = 2,
                  step = 1,
                  min = 8,
                  max = 128,
                  get = function() return _DB.Nameplates.height end,
                  set = function(_, height)
                    _DB.Nameplates.height = height ; Nameplate.defaultHeight = height
                  end,
                }
              }
            },
            position = {
              type = "group",
              name = "Position",
              inline = true,
              order = 3 ,
              args = {
                offsetX = {
                  type = "range",
                  name = "Offset X",
                  order = 1,
                  step = 1,
                  min = -300,
                  max = 300,
                  get = function() return _DB.Nameplates.offsetX end,
                  set = function(_, x)
                    _DB.Nameplates.offsetX = x ; Nameplate.defaultOffsetX = x
                  end
                },
                offsetY = {
                  type = "range",
                  name = "Offset Y",
                  order = 1,
                  step = 1,
                  min = -300,
                  max = 300,
                  get = function() return _DB.Nameplates.offsetY end,
                  set = function(_, y)
                    _DB.Nameplates.offsetY = y ; Nameplate.defaultOffsetY = y
                  end
                },
                anchorFrom = {
                  type = "select",
                  name = "Anchor from",
                  order = 4,
                  values = _AnchorPoints,
                  get = function() return _DB.Nameplates.anchorFrom end,
                  set = function(_, anchorPoint)
                    _DB.Nameplates.anchorFrom = anchorPoint ; Nameplate.defaultAnchorFrom = anchorPoint
                  end
                },
                anchorTo = {
                  type = "select",
                  name = "Anchor to",
                  order = 5,
                  values = _AnchorPoints,
                  get = function() return _DB.Nameplates.anchorTo end,
                  set = function(_, anchorPoint)
                    _DB.Nameplates.anchorTo = anchorPoint ; Nameplate.defaultAnchorTo = anchorPoint
                  end,
                }
              }
            }
          }
        },
        advanced = {
          type = "group",
          name = "Advanced",
          order = 2,
          args = {
            overlaps = {
              type = "group",
              name = "Overlap Settings",
              order = 2,
              inline = true,
              args = {
                vertical = {
                  type = "range",
                  name = "Vertical",
                  order = 1,
                  min = 0,
                  max = 5,
                  step = 0.05,
                  get = function() return tonumber(GetCVar("nameplateOverlapV")) end,
                  set = function(_, value) SetCVar("nameplateOverlapV", value) end,
                },
                horizontal = {
                  type = "range",
                  name = "Horizontal",
                  order = 2,
                  min = 0,
                  max = 5,
                  step = 0.05,
                  get = function() return tonumber(GetCVar("nameplateOverlapH")) end,
                  set = function(_, value) SetCVar("nameplateOverlapH", value) end,
                },
                toggleBackdropTest = {
                  type = "execute",
                  name = "Toggle Backdrop test",
                  order = 3,
                  func = function() _Addon:ToggleBackdropTest() end
                }
              }
            },
            clickThrought = {
              type = "group",
              name = "Click through",
              inline = true,
              order = 3,
              args = {
                clickThroughtFriendly = {
                  type = "toggle",
                  name = "Click through friendly nameplates",
                  order = 1,
                  width = "full",
                  -- C_NamePlate.GetNamePlateFriendlyClickThrough()
                  get = function() return _DB.Nameplates.clickThroughtFriendly end,
                  set = function(_, value) C_NamePlate.SetNamePlateFriendlyClickThrough(value); _DB.Nameplates.clickThroughtFriendly = value end,
                },
                clickThroughtEnemy = {
                  type = "toggle",
                  name = "Click through enemy nameplates",
                  order = 2,
                  width = "full",
                  -- C_NamePlate.GetNamePlateEnemyClickThrough()
                  get = function() return _DB.Nameplates.clickThroughtEnemy end,
                  set = function(_, value) C_NamePlate.SetNamePlateEnemyClickThrough(value); _DB.Nameplates.clickThroughtEnemy = value end,

                },
                clickThroughtSelf = {
                  type = "toggle",
                  name = "Click through self nameplate",
                  order = 3,
                  width = "full",
                  -- C_NamePlate.GetNamePlateSelfClickThrough()
                  get = function() return _DB.Nameplates.clickThroughtSelf end,
                  set = function(_, value) C_NamePlate.SetNamePlateSelfClickThrough(value); _DB.Nameplates.clickThroughtSelf = value end,
                }
              }
              --C_NamePlate.GetNamePlateFriendlyPreferredClickInsets
            },
            motion = {
              type = "group",
              name = "Motion",
              order = 4,
              inline = true,
              args = {
                simulateOverlapFriendlyDesc = {
                  type = "description",
                  name = [[Simulate the overlapping motion on the friendly nameplates even if your are in stacking motion.
It allows to have the enemy nameplates in stacking motion and the friendly nameplates in overlapping motion.
|cffff0000Warning : This option might not be supported by your nameplates addon. You need test by your own in order to see this one is compatible.|r]]

                },
                simulateOverlapFriendly  = {
                  type = "toggle",
                  name = "Simulate the overlapping motion on friendly nameplates",
                  order = 1,
                  width = "full",
                  get = function() return _DB.Nameplates.simulateOverlapMotionOnFriendly end,
                  set = function(_, value)
                    _DB.Nameplates.simulateOverlapMotionOnFriendly = value
                    if value then
                      _Addon:EnableSimulateOverlapMotionOnFriendly()
                    else
                      _Addon:DisableSimulateOverlapMotionOnFriendly()
                    end
                   end,
                }
              }
            }
          }
        }
      }
    }
  }
}

function OnLoad(self)
  _AceConfig:RegisterOptionsTable("BossModNameplates", _Settings)
  self:Open()
end

function OnEnable(self)

end

function OnDisable(self)

end

__SlashCmd__ "bmn" "config" "- open the options"
function Open()
  if not f then
    f = _AceGUI:Create("Frame")
  end
  _AceConfigDialog:Open("BossModNameplates", f)
end

-- @NOTE The users having an older scorpio version must force the loading.
-- Remove it later when more players have downloaded the last version.
if _SCORPIO_VERSION < 9 then

  _M:OnLoad()
end
